; ***************************************************************************************
; ***                                                                                 ***
; *** This code requires a modified DASM which was posted to [stella] at 02/19/2001 ! ***
; ***                                                                                 ***
; ***************************************************************************************

; *** DEATH DERBY ***

; 01/19/2003:
; - first working 2LK demo version with preliminary grenade
; 01/19/2003 (later):
; - improved grenade code:
;   - no safe zone colouring
;   - taler ball
;   - fullscreen mode switchable with right difficulty


    processor 6502
    include vcs.h

;===============================================================================
; T I A - C O N S T A N T S
;===============================================================================

BM_SIZE_1   = %000000
BM_SIZE_2   = %010000
BM_SIZE_4   = %100000
BM_SIZE_8   = %110000

HMOVE_L7    = $70
HMOVE_L6    = $60
HMOVE_L5    = $50
HMOVE_L4    = $40
HMOVE_L3    = $30
HMOVE_L2    = $20
HMOVE_L1    = $10
HMOVE_0     = $00
HMOVE_R1    = $f0
HMOVE_R2    = $e0
HMOVE_R3    = $d0
HMOVE_R4    = $c0
HMOVE_R5    = $b0
HMOVE_R6    = $a0
HMOVE_R7    = $90
HMOVE_R8    = $80

DISABLE_BM  = %00
ENABLE_BM   = %10


;===============================================================================
; U S E R - C O N S T A N T S
;===============================================================================

TOMBSTONEHEIGHT = 8  ; 8*2 = 16 (single scanlines) high per tombstone
TOMBSTONEROWS   = 11 ; 11 rows of tombstones per screen

CARHEIGHT       = 8  ; 8 doubled scanlines (used as offset)
ZOMBIE_H        = 8

CAR0_COL        = $36;
CAR1_COL        = $c6;

GRENADE_COL     = $58


;===============================================================================
; Z P - V A R I A B L E S
;===============================================================================

    SEG.U   variables
    ORG     $80

frameCnt    ds 1
tmpVar      ds 2
tmpVar2     = tmpVar + 1

krnlIdx     ds 1
vecKernel   ds 2

P0_Y        ds 1
P1_Y        ds 1
M0_Y        ds 1
M1_Y        ds 1
M0_Y2       ds 1
M1_Y2       ds 1

P0_X        ds 1
P1_X        ds 1
M0_X        ds 1
M1_X        ds 1
BL_X        ds 1

P0_YFullres ds 1
P1_YFullres ds 1
M0_YFullres ds 1
M1_YFullres ds 1

P0_Ptr      ds 2
P1_Ptr      ds 2
M0_Ptr      ds 2
M1_Ptr      ds 2

car0Id      ds 1
car1Id      ds 1
zombie0Id   ds 1
zombie1Id   ds 1

grenadeRow  ds 1
switchBL    ds 1        ; used for full screen ball


;===============================================================================
; M A C R O S
;===============================================================================

  MAC SLEEP
    IF {1} = 1
      ECHO "ERROR: SLEEP 1 not allowed !"
      END
    ENDIF
    IF {1} & 1
      nop $00
      REPEAT ({1}-3)/2
        nop
      REPEND
    ELSE
      REPEAT ({1})/2
        nop
      REPEND
    ENDIF
  ENDM


;===============================================================================
; R O M - C O D E
;===============================================================================

    SEG     Bank0
    ORG     $f000

Start SUBROUTINE
    sei                         ;           Disable interrupts, if there are any.
    cld                         ;           Clear BCD math bit.
    ldx     #0
    txs
    pha                         ;           Set stack to beginning.
    txa
.clearLoop:
    pha
    dex
    bne     .clearLoop

    jsr     GameInit

.mainLoop:
    jsr     VerticalBlank
    jsr     GameCalc
    jsr     DrawScreen
    jsr     OverScan
    jmp     .mainLoop


GameInit SUBROUTINE
    lda     #BM_SIZE_2|1
    sta     CTRLPF

    lda     #$00
    sta     COLUBK
    lda     #CAR0_COL   ;$36
    sta     COLUP0
    lda     #CAR1_COL   ;$c6
    sta     COLUP1

    lda     #64
    sta     P0_YFullres
    lda     #64
    sta     P1_YFullres

    lda     #16
    sta     P0_X
    lda     #136
    sta     P1_X

    lda     #100
    sta     M0_YFullres
    lda     #100
    sta     M1_YFullres

    lda     #80
    sta     M0_X
    lda     #88
    sta     M1_X

    lda     #50
    sta     BL_X

    lda     #>CarRotation0
    sta     P0_Ptr+1
    sta     P1_Ptr+1

    lda     #>Ped1_1
    sta     M0_Ptr+1
    sta     M1_Ptr+1

    ldx     #0
    stx     car0Id
    ldx     #8
    stx     car1Id

    ldx     #0
    stx     zombie0Id
    ldx     #3
    stx     zombie1Id

    lda     #3
    sta     grenadeRow

    lda     #1
    sta     VDELBL

    rts


VerticalBlank SUBROUTINE
    lda     #2
    sta     WSYNC
    sta     VSYNC
    sta     WSYNC
    sta     WSYNC
    lsr
    ldx     #44
    sta     WSYNC
    sta     VSYNC
    stx     TIM64T
    rts


GameCalc SUBROUTINE
    inc     frameCnt

; move the cars:
    lda     frameCnt
    and     #$03
    bne     .skipMoveP0
    ldx     P0_YFullres
    bne     .okP0
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+CARHEIGHT]*2
.okP0:
    dex
    stx     P0_YFullres
.skipMoveP0:

    lda     frameCnt
    and     #$07
    bne     .skipMoveP1
    ldx     P1_YFullres
    bne     .okP1
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+CARHEIGHT]*2
.okP1:
    dex
    stx     P1_YFullres
.skipMoveP1:

 ; move the zombies:
    lda     frameCnt
    and     #$03
    bne     .skipMoveM0
    ldx     M0_X
    bne     .okM0_X
    ldx     #159
.okM0_X:
    dex
    stx     M0_X

    ldx     M0_YFullres
    bne     .okM0_Y
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+ZOMBIE_H]*2
.okM0_Y:
    dex
    stx     M0_YFullres
.skipMoveM0

    lda     frameCnt
    and     #$07
    bne     .skipMoveM1
    ldx     M1_X
    bne     .okM1_X
    ldx     #159
.okM1_X:
    dex
    stx     M1_X

    ldx     M1_YFullres
    bne     .okM1_Y
    ldx     #[TOMBSTONEROWS*TOMBSTONEHEIGHT+ZOMBIE_H]*2
.okM1_Y:
    dex
    stx     M1_YFullres
.skipMoveM1

; move the ball:
    lda     frameCnt
    and     #$03
    bne     .skipMoveBL

    ldx     BL_X
    dex
    cpx     #17
    bcs     .okBL_X
    ldx     #143
.okBL_X:
    stx     BL_X
.skipMoveBL:


; *** calculate kernel variables: ***
    lda     P0_YFullres
    eor     #1
    sta     VDELP0
    eor     #1
    lsr
    sbc     #0
    clc
    adc     #CARHEIGHT-1
    sta     P0_Y

; calculate pointer to car graphics
    ldy     car0Id
    lda     CarTbl,y
;    sec
;    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     P0_Y
    sta     P0_Ptr

    lda     P1_YFullres
    sta     VDELP1
    lsr
    clc
    adc     #CARHEIGHT-1
    sta     P1_Y
    ldy     car1Id
    lda     CarTbl,y
;    sec
;    sbc     #TOMBSTONEROWS*TOMBSTONEHEIGHT
    clc
    adc     P1_Y
    sta     P1_Ptr


    lda     M0_YFullres
    lsr
    sta     M0_Y
    ldy     zombie0Id
    lda     Ped1_Tbl,y
;    clc
;    adc     #ZOMBIE_H-1
    sec
    sbc     M0_Y
    sta     M0_Ptr

    lda     M0_Y
    sec
    sbc     #ZOMBIE_H-1
    ora     #$80
    sta     M0_Y2

    ldy     #TOMBSTONEROWS*TOMBSTONEHEIGHT-1    ; zombie only partially visible?
    cpy     M0_Y
    bpl     .okM0
    sta     M0_Y
.okM0:

    ldy     M0_Y
    lda     (M0_Ptr),y
    asl
    asl
    sta     NUSIZ0


    lda     M1_YFullres
    lsr
    sta     M1_Y
    ldy     zombie1Id
    lda     Ped1_Tbl,y
;    clc
;    adc     #ZOMBIE_H-1
    sec
    sbc     M1_Y
    sta     M1_Ptr

    lda     M1_Y
    sec
    sbc     #ZOMBIE_H-1
    ora     #$80
    sta     M1_Y2

    ldy     #TOMBSTONEROWS*TOMBSTONEHEIGHT-1
    cpy     M1_Y
    bpl     .okM1
    sta     M1_Y
.okM1:

    ldy     M1_Y
    lda     (M1_Ptr),y
    asl
    asl
    sta     NUSIZ1


; position all objects
    ldx     #4
.loopPos:
    lda     P0_X,x
    jsr     SetPosX
    dex
    bpl     .loopPos
    sta     WSYNC

    sta     HMOVE
    sta     WSYNC

    sta     HMCLR
    rts


DrawScreen SUBROUTINE
; setting up a timer to make experimenting inside the kernel easier
; (the final code shouldm't need this):
    ldx     #228
.waitTim:
    lda     INTIM
    bne     .waitTim
    sta     WSYNC
    sta     VBLANK
    stx     TIM64T
    sta     WSYNC
;---------------------------------------

    SLEEP   22              ;22

    ldx     #TOMBSTONEROWS-1; 2
    ldy     #TOMBSTONEROWS*TOMBSTONEHEIGHT-1; 2

    lda     switchBL        ; 3
    sta     ENABL           ; 3

    lda     #$c0            ; 2                 enable left and right borders
    sta     PF0             ; 3

    lda     #$06            ; 2                 load color
    jmp     EnterKernel01   ; 3 = 42
;***************************************

    align   256

; wasting some bytes to avoid page penalties:
    ds      $74, 0

Kernel01 SUBROUTINE

.disableM0:                 ; 9
    SLEEP   2               ; 2                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM0a    ;10

.checkM0:                   ; 6
    bne     .disableM0      ; 2
;.enableM0:
    lda     M0_Y2           ; 3
    sta     M0_Y            ; 3
    lda     #ENABLE_BM      ; 2
.continueM0a:
    sta.w   ENAM0           ; 4         @75     one cycle too early, ok
    bpl     .continueM0     ; 3

.disableM1:                 ; 8
    SLEEP   2               ; 5                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM1a    ; 9

.checkM1:                   ; 6
    bne     .disableM1      ; 2
;.enableM1:
    lda     M1_Y2           ; 3
    sta     M1_Y            ; 3
    lda     #ENABLE_BM      ; 2
.continueM1a:
    sta.w   ENAM1           ; 4         @01
    bpl     .continueM1     ; 3

;***************************************

EnterKernel01:              ;

; cycle count:
; - loop overhead           13
; - COLUPF             3 =   3
; - playfield       28+8 =  36
; - players         18*2 =  36
; - missiles/HMOVE  23*2 =  46
; - HMCLR            3*2 =   6
; - free             6*2 =  12
;                          ---
; Total             76*2 = 152 :-)

.contBlock:                 ;           @42
    SLEEP   6               ; 6
    asl                     ; 2
    adc     #$00            ; 2 = 10            dummy!

.nextBlock:
    sta     COLUPF          ; 3 =  3    @55     too early for the ball

; draw zombie #0:
    cpy     M0_Y            ; 3
    bpl     .checkM0        ; 2
;.drawM0:
    lda     (M0_Ptr),y      ; 5
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     HMOVE           ; 3         @75     four cycles too early, but that's (just!) ok
;---------------------------------------
    sta     NUSIZ0          ; 3         @02
.continueM0:                ;   = 23            timing is constant!

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw1:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @21

; draw playfield:
    lda     PF1Data,x       ; 4
    sta     PF1             ; 3         @27
    lda     PF2Data,x       ; 4
    sta     PF2             ; 3         @34
    lda     PF1bData,x      ; 4
    sta     PF1             ; 3         @41
    lda     PF2bData,x      ; 4
    sta     PF2             ; 3 = 28    @48

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @51

    SLEEP   6

; draw zombie #1:
    cpy     M1_Y            ; 3
    bpl     .checkM1        ; 2
;.drawM1:
    lda     (M1_Ptr),y      ; 5
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     HMOVE           ; 3         @01     two cycles too early, ok
;---------------------------------------
    sta     NUSIZ1          ; 3         @04
.continueM1:                ;   = 23            timing is constant!

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0        ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw0:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @22

; disable playfield:
    lda     #0              ; 2
    sta     PF1             ; 3         @27
    sta     PF2             ; 3 =  8    @30

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @33

; TombstoneIndexer routine:
    dey                     ; 2
    tya                     ; 2
    and     #7              ; 2
    bne     .contBlock      ; 2
    dex                     ; 2
    bmi     .exitLoop       ; 2= 12

; enable grenade:
    lda     switchBL        ; 3
    cpx     grenadeRow      ; 3
    beq     .enableBL       ; 2
    .byte   $24             ; 1                 BIT_B
.enableBL:
    lsr                     ; 2
    sta     ENABL           ; 3 = 14    @59     VDELed!

; draw zombie #0:
    cpy     M0_Y            ; 3
    bpl     .checkM0_a      ; 2
;.drawM0_a:
    lda     (M0_Ptr),y      ; 5
    sta     HMM0            ; 3
    asl                     ; 2
    asl                     ; 2
;---------------------------------------
    sta     HMOVE           ; 3         @03     exact in time
    sta     NUSIZ0          ; 3         @06
.continueM0_a:              ;   = 23            timing is constant!

; draw car #1:
    lda     #CARHEIGHT-1    ; 2
    dcp     P1_Y            ; 5
    bcs     .doDraw1_a      ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw1_a:
    lda     (P1_Ptr),y      ; 5
    sta     GRP1            ; 3 = 18    @24

; set grenade color and clear PF0:
    lda     #GRENADE_COL    ; 2
    sta     COLUPF          ; 3         @29
    lda     #0              ; 2
    sta     PF0             ; 3 = 10    @34

; here would be time to copy another grenade into grenadeRow:
    SLEEP   20              ;20 = 20

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @57

; draw zombie #1:
    cpy     M1_Y            ; 3
    bpl     .checkM1_a      ; 2
;.drawM1_a:
    lda     (M1_Ptr),y      ; 5
    sta     HMM1            ; 3
    asl                     ; 2
    asl                     ; 2
    sta     HMOVE           ; 3         @01     two cycles too early, ok
;---------------------------------------
    sta     NUSIZ1          ; 3         @04
.continueM1_a:              ;   = 23            timing is constant!

; draw car #0:
    lda     #CARHEIGHT-1    ; 2
    dcp     P0_Y            ; 5
    bcs     .doDraw0_a      ; 2
    lda     #0              ; 2
    .byte   $2c             ;-1
.doDraw0_a:
    lda     (P0_Ptr),y      ; 5
    sta     GRP0            ; 3 = 18    @22

; disable grenade:
    lda     switchBL        ; 2
    sta     ENABL           ; 3         @28     VDELed!
    lda     #$c0            ; 2
    sta     PF0             ; 3 = 11    @33

; clear all hmov registers:
    sta     HMCLR           ; 3 =  3    @36

    SLEEP   9               ; 9 =  9

; TombstoneIndexer routine:
    dey                     ; 2
    lda     #$0e            ; 2
    jmp     .nextBlock      ; 3 =  7

.exitLoop:
    jmp     ExitKernel

;***************************************

.disableM0_a:               ; 9
    SLEEP   2               ; 2                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM0a_a  ;10

.checkM0_a:                 ; 6
    bne     .disableM0_a    ; 2
;.enableM0_a:
    lda     M0_Y2           ; 3
    sta     M0_Y            ; 3
    lda     #ENABLE_BM      ; 2
.continueM0a_a:
    sta.w   ENAM0           ; 4         @75     one cycle too early, ok
    bpl     .continueM0_a   ; 3

.disableM1_a:               ; 8
    SLEEP   2               ; 5                 waste two cycles for constant timing
    lda     #DISABLE_BM     ; 2
    beq     .continueM1a_a  ; 9

.checkM1_a:                 ; 6
    bne     .disableM1_a    ; 2
;.enableM1_a:
    lda     M1_Y2           ; 3
    sta     M1_Y            ; 3
    lda     #ENABLE_BM      ; 2
.continueM1a_a:
    sta.w   ENAM1           ; 4         @01
    bpl     .continueM1_a   ; 3
;***************************************


ExitKernel:
    sta     WSYNC
;---------------------------------------
    sta     PF0             ;                   a = 0
    sta     PF1
    sta     PF2
    sta     ENABL
    sta     GRP0
    sta     GRP1
    sta     GRP0
    sta     ENAM0
    sta     ENAM1

    ldx     #2
.waitScreen0:
    lda     INTIM
    bne     .waitScreen0
    sta     WSYNC
    stx     VBLANK
    rts


OverScan SUBROUTINE
    lda     #36
    sta     TIM64T

    lda     #%110
    bit     SWCHB
    bmi     .fullScreen
    asl
.fullScreen:
    sta     switchBL

;   ...

.waitTim:
    lda     INTIM
    bne     .waitTim
    rts


SetPosX SUBROUTINE
; TODO: use new Fatal Run method discovered by Manuel
; calculates the values and positions objects:
; Input:
; - a = x-position
    tay                      ; 2
    iny                      ; 2
    tya                      ; 2
    and    #$0F              ; 2
    sta    tmpVar            ; 3
    tya                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    lsr                      ; 2
    tay                      ; 2
    clc                      ; 2
    adc    tmpVar            ; 3
    cmp    #$0F              ; 2
    bcc    .skipIny          ; 2
    sbc    #$0F              ; 2
    iny                      ; 2
.skipIny:
    sta    WSYNC             ; 3
    eor    #$07              ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    asl                      ; 2
    sta.wx  HMP0,x           ; 5        @15
.waitPos:
    dey                      ; 2
    bpl    .waitPos          ; 2
.bankCheck:
    sta    RESP0,X           ; 4
    rts                      ; 6
;SetPosX END


;===============================================================================
; R O M - T A B L E S (Bank 0)
;===============================================================================
    org     $f700

PF1Data  ;D    76543210
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001
    .byte %00000000
    .byte %01111001
    .byte %01000101
    .byte %01000101
    .byte %01000101
    .byte %01111001

PF2Data  ;EA   01234567
    .byte %00101111
    .byte %10100000
    .byte %11100111
    .byte %00100000
    .byte %11101111
    .byte %00000000
    .byte %00101111
    .byte %00100000
    .byte %11100111
    .byte %00100000
    .byte %11101111

PF2bData ;     76543210;<--- scanning order
    .byte %11011110
    .byte %00010001
    .byte %10011110
    .byte %01010001
    .byte %10011110
    .byte %00000000
    .byte %01000100
    .byte %01000100
    .byte %11000100
    .byte %01000100
    .byte %11011111

PF1bData;      01234567;<--- scanning order
    .byte %10111110
    .byte %00100000
    .byte %10111110
    .byte %10100010
    .byte %10100010
    .byte %00000000
    .byte %00100010
    .byte %00100010
    .byte %00111110
    .byte %00100010
    .byte %00100010

CarTbl:
     .byte  <CarRotation0 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation1 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation2 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation3 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation4 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation5 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation6 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation7 - TOMBSTONEROWS*TOMBSTONEHEIGHT
     .byte  <CarRotation8 - TOMBSTONEROWS*TOMBSTONEHEIGHT

Ped1_Tbl:
    .byte   <Ped1_1 + ZOMBIE_H-1
    .byte   <Ped1_2 + ZOMBIE_H-1
    .byte   <Ped1_3 + ZOMBIE_H-1
    .byte   <Ped1_4 + ZOMBIE_H-1

    org     $f700 + TOMBSTONEROWS*TOMBSTONEHEIGHT

CarRotation0;UP (ACTUAL)
  ;.byte %00000000
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01100110; .XX..XX.
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...

CarRotation1
  ;.byte %00000000
  .byte %00000110; ....XX..
  .byte %00011110; ..XXXX..
  .byte %01110100; XXX.X...
  .byte %00011010; XX.XX.X.
  .byte %00011010; ...XX.X.
  .byte %01011111; .X.XXXXX
  .byte %00111100; ..XXXX..
  .byte %00101100; ..X.XX..

CarRotation2
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01111000; .XXXX...
  .byte %01000010; .X....X.
  .byte %11011011; XX.XX.XX
  .byte %11011100; XX.XXX..
  .byte %00011110; ...XXXX.
  .byte %00110110; ..XX.XX.
  .byte %00010000; ...X....

CarRotation3
  ;.byte %00000000
  .byte %00110000; ..XX....
  .byte %00110010; ..XX.X..
  .byte %01100011; .XX...XX
  .byte %01011110; .X.XXXX.
  .byte %11111111; XXXXXXXX
  .byte %11000111; XX...XXX
  .byte %00011100; ...XXX..
  .byte %00000100; .....X..

CarRotation4;RIGHT|LEFT
  ;.byte %00000000
  .byte %00000000; ........
  .byte %11101110; XXX.XXX.
  .byte %01100100; .XX..X..
  .byte %11011111; XX.XXXXX
  .byte %11011111; XX.XXXXX
  .byte %01100100; .XX..X..
  .byte %11101110; XXX.XXX.
  .byte %00000000; ........

CarRotation5
  ;.byte %00000000
  .byte %00000100; .....X..
  .byte %00011100; ...XXX..
  .byte %11000111; XX...XXX
  .byte %11111111; XXXXXXXX
  .byte %01011110; .X.XXXX.
  .byte %01100011; .XX...XX
  .byte %00110010; ..XX.X..
  .byte %00110000; ..XX....

CarRotation6
  ;.byte %00000000
  .byte %00010000; ...X....
  .byte %00110110; ..XX.XX.
  .byte %00011110; ...XXXX.
  .byte %11011100; XX.XXX..
  .byte %11011011; XX.XX.XX
  .byte %01000010; .X....X.
  .byte %01111000; .XXXX...
  .byte %00011000; ...XX...

CarRotation7
  ;.byte %00000000
  .byte %00101100; ..X.XX..
  .byte %00111100; ..XXXX..
  .byte %01011111; .X.XXXXX
  .byte %00011010; ...XX.X.
  .byte %11011010; XX.XX.X.
  .byte %11101000; XXX.X...
  .byte %00111100; ..XXXX..
  .byte %00001100; ....XX..

CarRotation8;DOWN (ACTUAL)
  ;.byte %00000000
  .byte %00011000; ...XX...
  .byte %01011010; .X.XX.X.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.
  .byte %00011000; ...XX...
  .byte %01100110; .XX..XX.
  .byte %01111110; .XXXXXX.
  .byte %01011010; .X.XX.X.

   .byte HMOVE_0
Ped1_1;   OOOOWW__
   .byte HMOVE_L1|BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_L1|BM_SIZE_2/4 ; ..XX....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R2|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L3|BM_SIZE_4/4 ; .XXXX...
   .byte HMOVE_R1|BM_SIZE_1/4 ; ....X...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XX...

Ped1_2;   OOOOWW__
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

Ped1_3;   OOOOWW__
   .byte %11110100 ; .XX.....
   .byte %00000000 ; ..X.....
   .byte %11011100 ; ...XXXX.
   .byte %11100100 ; ..XX....
   .byte %11010100 ; ...XX...
   .byte %00000000 ; ........
   .byte %11010100 ; ...XX...
   .byte %11010100 ; ...XX...

   .byte HMOVE_0
Ped1_4;   OOOOWW__
   .byte HMOVE_0 |BM_SIZE_2/4 ; ...XXXX.
   .byte HMOVE_0 |BM_SIZE_1/4 ; ...X....
   .byte HMOVE_0 |BM_SIZE_4/4 ; ...XXXX.
   .byte HMOVE_R3|BM_SIZE_2/4 ; ...XX...
   .byte HMOVE_L2|BM_SIZE_4/4 ; XXXX....
   .byte HMOVE_R1|BM_SIZE_4/4 ; ..XXXX..
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....
   .byte HMOVE_0 |BM_SIZE_2/4 ; .XX.....

    org $f7fc
    .word   Start
    .word   Start
